"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Promise = require("bluebird");
var coffeelint = require('coffeelint');
var reporter = require('coffeelint/lib/reporters/default');
var depcheck = require("depcheck");
var fs = require("fs");
var glob = require("glob");
var merge = require('merge');
var optimist = require("optimist");
var path = require("path");
var tslint = require("tslint");
var configurations = {
    coffeescript: {
        configPath: path.join(__dirname, '../config/coffeelint.json'),
        configFileName: 'coffeelint.json',
        extensions: ['coffee'],
        lang: 'coffeescript',
    },
    typescript: {
        configPath: path.join(__dirname, '../config/tslint.json'),
        configFileName: 'tslint.json',
        extensions: ['ts', 'tsx'],
        lang: 'typescript',
    },
};
var getPackageJsonDir = function (dir) {
    var name = findFile('package.json', dir);
    if (name === null) {
        throw new Error('Could not find package.json!');
    }
    return path.dirname(name);
};
var read = function (path) {
    var realPath = fs.realpathSync(path);
    return fs.readFileSync(realPath).toString();
};
var findFile = function (name, dir) {
    dir = dir || process.cwd();
    var filename = path.join(dir, name);
    var parent = path.dirname(dir);
    if (fs.existsSync(filename)) {
        return filename;
    }
    else if (dir === parent) {
        return null;
    }
    else {
        return findFile(name, parent);
    }
};
var parseJSON = function (file) {
    try {
        return JSON.parse(fs.readFileSync(file).toString());
    }
    catch (err) {
        console.error("Could not parse " + file);
        throw err;
    }
};
var findFiles = function (extensions, paths) {
    if (paths === void 0) { paths = []; }
    var files = [];
    for (var _i = 0, paths_1 = paths; _i < paths_1.length; _i++) {
        var p = paths_1[_i];
        if (fs.statSync(p).isDirectory()) {
            files = files.concat(glob.sync(p + "/**/*.@(" + extensions.join('|') + ")"));
        }
        else {
            files.push(p);
        }
    }
    return files.map(function (p) { return path.join(p); });
};
var lintCoffeeFiles = function (files, config) {
    var errorReport = new coffeelint.getErrorReport();
    for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
        var file = files_1[_i];
        var source = read(file);
        errorReport.lint(file, source, config);
    }
    var report = new reporter(errorReport, {
        colorize: process.stdout.isTTY,
        quiet: false,
    });
    report.publish();
    return errorReport.getExitCode();
};
var lintTsFiles = function (files, config) {
    var parsedConfig = tslint.Configuration.parseConfigFile(config);
    var linter = new tslint.Linter({
        fix: false,
        formatter: 'stylish',
    });
    for (var _i = 0, files_2 = files; _i < files_2.length; _i++) {
        var file = files_2[_i];
        var source = read(file);
        linter.lint(file, source, parsedConfig);
    }
    var errorReport = linter.getResult();
    console.log(linter.getResult().output);
    return errorReport.errorCount === 0 ? 0 : 1;
};
var runLint = function (resinLintConfig, paths, config) {
    var linterExitCode;
    var scripts = findFiles(resinLintConfig.extensions, paths);
    if (resinLintConfig.lang === 'typescript') {
        linterExitCode = lintTsFiles(scripts, config);
    }
    if (resinLintConfig.lang === 'coffeescript') {
        linterExitCode = lintCoffeeFiles(scripts, config);
    }
    return process.on('exit', function () { return process.exit(linterExitCode); });
};
exports.lint = function (passedParams) {
    try {
        var options_1 = optimist(passedParams)
            .usage('Usage: resin-lint [options] [...]')
            .describe('f', 'Specify a linting config file to override resin-lint rules')
            .describe('p', 'Print default resin-lint linting rules')
            .describe('i', 'Ignore linting config files in project directory and its parents')
            .boolean('typescript', 'Lint typescript files instead of coffeescript')
            .boolean('u', 'Run unused import check');
        if ((options_1.argv._.length < 1) && !options_1.argv.p) {
            options_1.showHelp();
            process.exit(1);
        }
        return Promise.try(function () {
            if (options_1.argv.u) {
                return Promise.map(options_1.argv._, function (dir) {
                    dir = getPackageJsonDir(dir);
                    return Promise.resolve(depcheck(path.resolve('./', dir), {
                        ignoreMatches: [
                            '@types/*',
                            'supervisor',
                            'coffee-script',
                            'coffeescript',
                            'colors',
                            'coffeescope2',
                        ],
                    }))
                        .get('dependencies')
                        .then(function (deps) {
                        if (deps.length > 0) {
                            console.log(deps.length + " unused dependencies:");
                            for (var _i = 0, deps_1 = deps; _i < deps_1.length; _i++) {
                                var dep = deps_1[_i];
                                console.log("\t" + dep);
                            }
                            process.exit(1);
                        }
                        console.log('No unused dependencies!');
                        return console.log();
                    });
                });
            }
        }).then(function () {
            var configOverridePath;
            var resinLintConfiguration = options_1.argv.typescript ? configurations.typescript : configurations.coffeescript;
            if (options_1.argv.p) {
                console.log(fs.readFileSync(resinLintConfiguration.configPath).toString());
                process.exit(0);
            }
            var config = parseJSON(resinLintConfiguration.configPath);
            if (options_1.argv.f) {
                configOverridePath = fs.realpathSync(options_1.argv.f);
            }
            if (!options_1.argv.i && !configOverridePath) {
                configOverridePath = findFile(resinLintConfiguration.configFileName);
            }
            if (configOverridePath) {
                var configOverride = parseJSON(configOverridePath);
                config = merge.recursive(config, configOverride);
            }
            var paths = options_1.argv._;
            return runLint(resinLintConfiguration, paths, config);
        }).return();
    }
    catch (err) {
        return console.log(err.stack);
    }
};
//# sourceMappingURL=resin-lint.js.map